#############
# $Id: archive.pm 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-09
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/lib/npg_pipeline/archive.pm $

package npg_pipeline::archive;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12581 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::base};
with qw{npg_common::roles::run::path_info};
with qw{npg_common::roles::run::long_info};

Readonly::Scalar our $UNIQUE_SCRIPT_RUNNER        => q{script_must_be_unique_runner};

has q{inst_name}      => (isa => q{Str}, is => q{ro}, lazy_build => 1);

sub change_to_archive_directory {
  my ($self) = @_;
  my $archive_dir = $self->archive_path();
  chdir "$archive_dir" or croak "could not cd to $archive_dir";
  return 1;
}

### private methods

sub _unique_script_runner {
  my ($self) = @_;
  return $UNIQUE_SCRIPT_RUNNER;
}

sub _build_inst_name {
  my ($self) = @_;
  return $self->instrument_string();
}
no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

  my $archive_object = npg_pipeline::archive::<sub_class>->new({
    run_folder    => $sRunFolder, # required
    id_run        => $iIdRun,
    analysis_clas => $sAnalysisPath,
  });

  if analysis_path is given, then this should be the path to the GERALD dir that you want to use

=head1 DESCRIPTION

Ultimate base class for objects which do stuff on files (create,read or update) files in the archive directory

=head1 SUBROUTINES/METHODS

=head2 change_to_archive_directory - changes the directory we are currently working in to the archive directory

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Cwd

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
