#############
# $Id: correct_qseq.pm 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-08-26
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/lib/npg_pipeline/archive/file/correct_qseq.pm $

package npg_pipeline::archive::file::correct_qseq;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12581 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::archive::file::generation};
with qw{npg_pipeline::roles::business::tag_info};

Readonly::Scalar our $PROGRAM_FILE  => q{index_decoder};

sub correct {
  my ($self, $arg_refs) = @_;
  my $id_run = $arg_refs->{id_run};

  my $use_bases = $self->use_bases();

  if ($use_bases !~ /i\d+/ixms) {
    return ();
  }

  $self->_create_qseq_lane_dirs($arg_refs->{owning_group});
  my @jids;
  foreach my $lane (1..$self->max_lane()) {
    $arg_refs->{lane} = $lane;
    my $job_sub = $self->_generate_bsub_command($arg_refs);
    my $jid = $self->submit_bsub_command($job_sub);
    push @jids, $jid;
  }

  return @jids;
}

sub _create_qseq_lane_dirs {
  my ($self, $owning_group) = @_;

  $owning_group ||= $ENV{OWNING_GROUP};
  $owning_group ||= $self->owning_group();

  if ( ! -d $self->tag_qseqs_path() ) {
    $self->log( q{creating } . $self->tag_qseqs_path() );
    my $mkdir_cmd = q{mkdir -p } . $self->tag_qseqs_path();
    my $rc = `$mkdir_cmd`;
    if (!$rc != 1) {
      croak q{could not create } . $self->tag_qseqs_path();
    }
    my $chgrp_cmd = qq{chgrp $owning_group } . $self->tag_qseqs_path();
    $self->log( $chgrp_cmd );
    $rc = `$chgrp_cmd`;
    if (!$rc != 1) {
      $self->log( q{could not chgrp } . $self->tag_qseqs_path() );                # not fatal
    }
  } else {
    $self->log($self->tag_qseqs_path() . q{ already exists});
  }

  return 1;
}

sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();
  my $lane = $arg_refs->{lane} || q{};

  my $tag_qseqs_dir = $self->tag_qseqs_path();

  my $job_name = q{tag_qseqs_} . $self->id_run() . q{_} . $lane . q{_} . $timestamp;

  my $qseq_file_string = $self->qseq_file_constructs(q{all_lanes});

  my @qseqs = split /[ ]/xms, $qseq_file_string;

  my @wanted_qseqs;

  foreach my $qseq (@qseqs) {
    if ($qseq && $qseq =~ m{/s_$lane}xms) {
      push @wanted_qseqs, $qseq;
    }
  }

  $qseq_file_string = join q{ }, @wanted_qseqs;
  my $use_bases = $self->use_bases();
  my ($index_length) = $use_bases =~ /I(\d)/xms;
  my $tag_file = $self->get_tag_file({
      index_length => $index_length,
    });

  my $bsub_command = q{bsub -q } . $self->lsf_queue() . qq{ $required_job_completion -J $job_name };
  $bsub_command .=  ( $self->fs_resource_string( {
    counter_slots_per_job => 1,
  } ) ) . q{ };
  $bsub_command .=  q{-o } . $tag_qseqs_dir . qq{/$job_name.out };
  $bsub_command .=  q{'} . $self->internal_script_bin() . q{/} . $self->external_script_names_conf()->{index_decoder} . q{ };
  $bsub_command .= qq{-t $tag_file };
  $bsub_command .=  q{-u } . $use_bases . q{ };
  $bsub_command .= qq{-o $tag_qseqs_dir };
  $bsub_command .= qq{$qseq_file_string };
  $bsub_command .= qq{> $tag_qseqs_dir/} . $self->external_script_names_conf()->{index_decoder} . q{_}. $lane .q{.out'};
  if ($self->verbose()) {
    $self->log($bsub_command);
  }
  return $bsub_command;
}


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::correct_qseq

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 correct

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
