#############
# $Id: harold_analysis_runner.pm 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-09-14
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/lib/npg_pipeline/daemons/harold_analysis_runner.pm $

package npg_pipeline::daemons::harold_analysis_runner;
use Moose;
use MooseX::InsideOut;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12581 $ =~ /(\d+)/mxs; $r; };
use npg::api::run_status_dict;
use npg::api::run;

extends qw{npg_pipeline::daemons};

Readonly::Scalar our $ANALYSIS_SCRIPT   => q{npg_pipeline_analyse_RTA};
Readonly::Scalar our $PB_CAL_SCRIPT     => q{npg_pipeline_PB_cal};

Readonly::Scalar our $PB_CAL_NO_LATEST_SUMMARY_OPTIONS => q{--no_munge_summary --no_summary_link --no_status_updates};
Readonly::Scalar our $PB_CAL_LATEST_SUMMARY_OPTIONS => q{--no_munge_summary};
Readonly::Scalar our $PB_CAL_SPIKED_OPTIONS => $PB_CAL_LATEST_SUMMARY_OPTIONS;
Readonly::Scalar our $OPTIONS_IF_SPIKED_ANALYSIS_GOING => q{--no_summary_link --no_status_updates};

Readonly::Scalar our $ANALYSIS_PENDING  => q{analysis pending};

sub run {
  my ($self) = @_;

  #################
  # obtain runs with a status of analysis pending
  my $runs_to_process = npg::api::run_status_dict->new({
    description => $ANALYSIS_PENDING,
    ($self->_has_api_util() ? (q{util} => $self->_api_util()) : ()),
  })->runs();

  #################
  # go through each run, running the npg_pipeline_analyse_RTA script on each
  # run on first instance of seeing an end of the run, and skipping the other end

  my $seen = $self->_seen();

  my $default_queue = $ENV{LSB_DEFAULTQUEUE} || q{(generic default)};

  $self->log(q{Default Queue is } . $default_queue);

  foreach my $run ( @{ $runs_to_process } ) {

    my $id_run = $run->id_run();
    # skip if you have already seen this
    if ($seen->{$id_run}) {
      $self->log(qq{Already seen $id_run});
      next;
    }

#    my $run = npg::api::run->new( {
#      id_run => $id_run,
#      ($self->_has_api_util() ? (q{util} => $self->_api_util()) : ()),
#    } );

    my $inst_model = $run->instrument->model();

    # only run if this run is an rta run
    if ( $run->is_rta() || $self->_is_rta() ) {

      my $run_is_spiked;
      foreach my $lane ( @{ $run->batch->lanes() } ) {
        if ( $lane->is_spiked_phix() ) {
          $run_is_spiked++;
        }
      }

      my $arg_refs = {
        id_run => $id_run,
        script => $PB_CAL_SCRIPT,
        inst_model => $inst_model,
      };

      # launch full pb_cal version - this will not be used for production results though at this time
      $arg_refs->{options} = $PB_CAL_NO_LATEST_SUMMARY_OPTIONS;
      my $cmd = $self->_generate_command( $arg_refs );

#      eval {
#        $self->_run_command( $id_run, $cmd );
#      } or do {
#        $self->log( $EVAL_ERROR );
#        next;
#      };

      if ( $run_is_spiked ) {
        $arg_refs->{options} = $PB_CAL_SPIKED_OPTIONS;
        $cmd = $self->_generate_command( $arg_refs );

        eval {
          $self->_run_command( $id_run, $cmd);
        } or do {
          $self->log( $EVAL_ERROR );
          next;
        };
      }

      if ( $inst_model eq q{HiSeq} ) {

        # launch no recalibration pb_cal version - this will be used for production results at this time
        $arg_refs->{options} = $PB_CAL_LATEST_SUMMARY_OPTIONS . q{ --no_recalibration};

        if ( $run_is_spiked ) {
          $arg_refs->{options} .= q{ } . $OPTIONS_IF_SPIKED_ANALYSIS_GOING;
        }

        $cmd = $self->_generate_command( $arg_refs );

        eval {
          $self->_run_command( $id_run, $cmd );
        } or do {
          $self->log( $EVAL_ERROR );
          next if ! $run_is_spiked;
        };

      } else {

        # launch standard RTA analysis - this willbe used for production results at this time
        $arg_refs->{script} = $ANALYSIS_SCRIPT;
        $arg_refs->{options} = q{};

        if ( $run_is_spiked ) {
          $arg_refs->{options} .= q{ } . $OPTIONS_IF_SPIKED_ANALYSIS_GOING;
        }

        $cmd = $self->_generate_command( $arg_refs );

        eval {
          $self->_run_command( $id_run, $cmd );
        } or do {
          $self->log( $EVAL_ERROR );
          next if ! $run_is_spiked;
        };

      }

      $seen->{$id_run}++; # you have now seen this

    } else {

      $self->log(qq{$id_run is not an rta run});
      $seen->{$id_run}++; # you have now seen this

    }

  }

  return 1;
}

##############
# Private methods

# generate the command to run
sub _generate_command {
  my ( $self, $arg_refs ) = @_;

  my $id_run  = $arg_refs->{id_run};
  my $script  = $arg_refs->{script};
  my $options = $arg_refs->{options};
  my $inst_model = $arg_refs->{inst_model};

  return $self->all_source_files_as_a_source_string() . $self->internal_script_bin() . q{/} . $script . q{ --verbose --id_run } . $id_run . q{ --instrument_model } . $inst_model . q{ } . $options;
}

# run the command generated
sub _run_command {
  my ( $self, $id_run, $cmd ) = @_;

  $self->log(qq{Running $cmd});

  my $output = `$cmd`;

  if ( $CHILD_ERROR ) {
    croak qq{Error $CHILD_ERROR occured. Will try $id_run again on next loop.}
  };

  $self->log(qq{Output:\n$output});

  return 1;
}

# manual override to determine if run is rta - usually for testing
has q{_is_rta} => (
  isa => q{Bool}, is => q{ro},
);


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::daemons::harold_analysis_runner

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

  my $runner = npg_pipeline::daemons::harold_analysis_runner->new();
  $runner->run();

=head1 DESCRIPTION

This module interrogates the npg database for runs with a status of analysis pending, and that are rta runs, and then runs the npg_pipeline_analyse_RTA script on each of them

=head1 SUBROUTINES/METHODS

=head2 run - the only method and the only one you need. It does everything.

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_pipeline::daemons

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
