#############
# $Id: dispatch_tree.pm 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-11-02
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/lib/npg_pipeline/roles/generic/dispatch_tree.pm $

package npg_pipeline::roles::generic::dispatch_tree;
use Moose::Role;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12581 $ =~ /(\d+)/mxs; $r; };

=head1 NAME

npg_pipeline::roles::generic::dispatch_tree

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::generic::dispatch_tree};

=head1 DESCRIPTION

Imports the methods needed to utilise the dispatch tree object

=head1 SUBROUTINES/METHODS

=head2 dispatch_tree - Do not set this on construction. It builds and utilises a npg_pipeline::dispatch_tree object,
which internally stores the job tree dispatched. This tree can then be logged into the log file at the end

  my $oDispatchTree = $class->dispatch_tree();

=cut

has q{dispatch_tree}      => (isa => q{Ref}, is => q{ro}, lazy_build => 1,  documentation => q{Do Not set on construction, bad things may happen});

sub _build_dispatch_tree {
  my ($self) = @_;
  return npg_pipeline::dispatch_tree->new({
    schema => __PACKAGE__,
    id_run => $self->id_run(),
    timestamp => $self->timestamp(),
    script_name => $self->script_name(),
    run_folder => $self->run_folder(),
  });
}


1;
__END__


=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
