#!/Users/ajb/dev/bin/perl -w

# $Id: functional_script.pl 12581 2011-02-11 12:26:13Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-06-11
# Last Changed On: $Date: 2011-02-11 12:26:13 +0000 (Fri, 11 Feb 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-25.0/scripts/functional_script.pl $

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Getopt::Long;
use lib qw{lib};

my $opts = {};
GetOptions($opts, qw{id_run=s lane=s completed_function=s schema=s alert=s other_params=s});

if (!$opts->{schema}) {
  croak qq{Usage $opts->{script_name} -schema=<schema_name> (-id_run=<id_run> -lane=<lane> -alert=<email> -other_params=<string_of_other_params>)\n\nIf passing 'other_params', ensure that your code in the schema module knows how to translate it\n\n};
}

$opts->{script_name} = q{functional_script.pl};
$opts->{completed_function} ||= q{none};

if ($opts->{alert}) { ($opts->{alert}) = $opts->{alert}  =~ /([[:alpha:]\d_\.@]+)/xms; }

my ($schema)     = $opts->{schema} =~ /([[:alpha:]\d_]+)/xms;

my $schema_module_file = q{npg_common/pluggable/}.$opts->{schema}.q{.pm};
my $schema_module = q{npg_common::pluggable::}.$opts->{schema};
require $schema_module_file;
my $pluggable = $schema_module->new($opts);
$pluggable->main();
0;

__END__

=head1 NAME

  functional_script.pl

=head1 VERSION

$LastChangedRevision: 12581 $

=head1 SYNOPSIS

  Usage functional_script.pl -schema=<schema_name> (-id_run=<id_run> -lane=<lane> -alert=<email> -other_params=<string_of_other_params>)
  Usage functional_script.pl -schema=<schema_name> -completed_function=<last_function_completed> (-id_run=<id_run> -lane=<lane> -alert=<email> -other_params=<string_of_other_params>)

=head1 DESCRIPTION

This script is designed as a starter for running a pluggable schema system, by taking parameters it loads the schema object, and passes through the parameters.
The schema object is responsible for knowing which function to run based on the last function completed in it's main method

=head1 SUBROUTINES/METHODS

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item strict

=item warnings

=item Carp

=item English -no_match_vars

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 Andy Brown (setitesuk@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
