# $Id: 20-archive_file-to_irods.t 12581 2011-02-11 12:26:13Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 11;
use Test::Exception;
use lib qw{t};
use t::util;
use npg::api::run;

use_ok('npg_pipeline::archive::file::to_irods');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_API_UTIL} = $util->apiutil();
$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
my $tmp_dir = $util->temp_directory();

my $analysis_runfolder_path = $util->analysis_runfolder_path();

sub create_analysis {
  `rm -rf $tmp_dir/staging`;
  `mkdir -p $analysis_runfolder_path/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive`;
  `mkdir $analysis_runfolder_path/Config`;
  `cp t/data/Recipes/Recipe_GA2_37Cycle_PE_v6.1.xml $analysis_runfolder_path/`;
  `cp t/data/Recipes/TileLayout.xml $analysis_runfolder_path/Config/`;
  `ln -s Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto $analysis_runfolder_path/Latest_Summary`;

  my $archive_root = qq{$analysis_runfolder_path/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/};
  foreach my $i (1..7) {
    foreach my $extension (qw{bam}) {
      my $file = $archive_root . qq{1234_$i}.qq{.$extension};
      `touch $file`;
    }
  }

  `touch $archive_root/1234_8_human.bam`;
  `touch $archive_root/1234_8_nonhuman.bam`;

  return 1;
}

my $run_obj = npg::api::run->new({
  util   => $util->apiutil(),
  id_run => 1234,
 });

{
  my $bam_irods;

  lives_ok { $bam_irods = npg_pipeline::archive::file::to_irods->new({
    run_folder => q{123456_IL2_1234},
    run => $run_obj,
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{created with run_folder ok};
  isa_ok($bam_irods , q{npg_pipeline::archive::file::to_irods}, q{object test});
  create_analysis();

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my @jids;
  lives_ok { @jids = $bam_irods->submit_to_lsf($arg_refs); } q{no croak submitting job to lsf};

  is(scalar@jids, 1, q{only one job submitted});

  my $bsub_command = $util->drop_temp_part_from_paths( $bam_irods ->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -w'done(123) && done(321)' -J irods_bam_loader.pl_1234_20090709-123456 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/log/irods_bam_loader.pl_1234_20090709-123456.out 't/bin/software/solexa/bin/irods_bam_loader.pl --archive_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --runfolder_path /staging/IL2/analysis/123456_IL2_1234 --id_run 1234'};

  is( $bsub_command, $expected_command, q{generated bsub command is correct});
}

{
  my $bam_irods;

  lives_ok { $bam_irods = npg_pipeline::archive::file::to_irods->new({
    run_folder => q{123456_IL2_1234},
    run => $run_obj,
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
    lanes      => [8],
  }); } q{created with run_folder ok};
  isa_ok($bam_irods , q{npg_pipeline::archive::file::to_irods}, q{object test});
  create_analysis();

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my @jids;
  lives_ok { @jids = $bam_irods->submit_to_lsf($arg_refs); } q{no croak submitting job to lsf};

  is(scalar@jids, 1, q{only one job submitted});

  my $bsub_command = $util->drop_temp_part_from_paths( $bam_irods ->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -w'done(123) && done(321)' -J irods_bam_loader.pl_1234_20090709-123456 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/log/irods_bam_loader.pl_1234_20090709-123456.out 't/bin/software/solexa/bin/irods_bam_loader.pl --archive_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --runfolder_path /staging/IL2/analysis/123456_IL2_1234 --id_run 1234 --positions 8'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct} );
}

1;
__END__
