# $Id: 20-archive_file-to_sra.t 12581 2011-02-11 12:26:13Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 14;
use Test::Exception;
use lib qw{t};
use t::util;
use File::Slurp;

use_ok('npg_pipeline::archive::file::to_sra');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
my $tmp_dir = $util->temp_directory();

my $analysis_runfolder_path = $util->analysis_runfolder_path();
$ENV{TEST_USERAGENT} = $analysis_runfolder_path . q{/useragent};

sub create_analysis {
  `rm -rf $tmp_dir/staging`;
  `mkdir -p $analysis_runfolder_path/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive`;
  `mkdir $analysis_runfolder_path/Config`;
  `cp t/data/Recipes/Recipe_GA2_37Cycle_PE_v6.1.xml $analysis_runfolder_path/`;
  `cp t/data/Recipes/TileLayout.xml $analysis_runfolder_path/Config/`;
  `ln -s Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto $analysis_runfolder_path/Latest_Summary`;

  my $archive_root = qq{$analysis_runfolder_path/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/};
  foreach my $i (1..8) {
    foreach my $extension (qw{fastq fastqcheck gcfreq md5}) {
      my $file = $archive_root . qq{1234_$i}.qq{.$extension};
      `touch $file`;
    }
  }

  `touch $archive_root/1234_7_human.fastq`;
  `touch $archive_root/1234_7_nonhuman.fastq`;

  return 1;
}

{
  my $fsa;

  lives_ok { $fsa = npg_pipeline::archive::file::to_sra->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{created with run_folder ok};
  isa_ok($fsa, q{npg_pipeline::archive::file::to_sra}, q{$fsa});
  create_analysis();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my @jids;
  lives_ok { @jids = $fsa->submit_to_lsf($arg_refs); } q{no croak submitting job to lsf};

  is(scalar@jids, 1, q{only one job submitted});

  my $bsub_command = $util->drop_temp_part_from_paths( $fsa->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -w'done(123) && done(321)' -J npg_pipeline_archive_to_sra_1234_20090709-123456 -R 'rusage[nfs_12=1]' -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/log/npg_pipeline_archive_to_sra_1234_20090709-123456.out 't/bin/software/solexa/bin/npg_pipeline_archive_to_sra --basecall_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls --bustard_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto --conf_path data/config_files --data_path /staging/IL2/analysis/123456_IL2_1234/Data --domain test --fs_resource nfs_12 --id_run 1234 --intensity_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto --internal_script_bin t/bin/software/solexa/bin --lsf_queue test --name IL2_1234 --pb_cal_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/PB_cal --recalibrated_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto --run_folder 123456_IL2_1234 --runfolder_path /staging/IL2/analysis/123456_IL2_1234 --subpath  --tag_qseqs_path /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/tag_qseqs --verbose '};
  is($bsub_command, $expected_command, q{generated bsub command is correct});
}

{
  create_analysis();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );
  my $fsa = npg_pipeline::archive::file::to_sra->new({
    run_folder => q{123456_IL2_1234},
    conf_path => $conf_path,
    domain => q{test},
  });
  ok($fsa->archive(), q{true value returned from running $fsa->archive()});

  my $hash = {
  };

  $fsa->_lane_files_md5_vals_for_test($hash);
  $fsa->_test_md5(q{md5});
  lives_ok { $fsa->archive(); } q{md5sum ok};

  $hash->{6}->{q[4350_6.fastq]} = q[222ba4d1344e21ff833c02fe42fcspam];
  throws_ok { $fsa->archive(); } qr{4350_6[.]fastq[ ]has[ ]incorrect[ ]md5:\n\tFilesystem:\t222ba4d1344e21ff833c02fe42fcspam\n\tFuse:}, q{md5sum not ok - croak};

}

{
  create_analysis();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );
  lives_ok{ qx[bin/npg_pipeline_archive_to_sra --conf_path $conf_path --domain test --internal_script_bin t/bin/software/solexa/bin --verbose  --subpath  --basecall_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls --run_folder 123456_IL2_1234 --intensity_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto --name IL2_1234 --data_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data --bustard_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto --id_run 1234 --runfolder_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234 --recalibrated_path $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto] } q{npg_pipeline_archive_to_sra ran};
  ok(!$CHILD_ERROR, qq{Error code ok : $CHILD_ERROR});
}
{
  create_analysis();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );
  lives_ok{ qx[bin/npg_pipeline_archive_to_sra --conf_path $conf_path --domain test --verbose  --run_folder 123456_IL2_1234 --name IL2_1234 --id_run 1234] } q{npg_pipeline_archive_to_sra ran};
  ok(!$CHILD_ERROR, qq{Error code ok : $CHILD_ERROR});
}
$ENV{TEST_DIR} = undef;
{
  my $fsa = npg_pipeline::archive::file::to_sra->new({
    run_folder => q{123456_IL2_1234},
    conf_path => $conf_path,
    domain => q{test},
  });
  is( $fsa->fuse_md5_archive(), q{t/fuse/mpsafs/runs/1234/.md5}, q{fuse_md5_archive location is correct});
}
