# $Id: 20-archive_file-to_sra_hiseq.t 12581 2011-02-11 12:26:13Z ajb $
use strict;
use warnings;
use Test::MockObject;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 4;
use Test::Exception;
use Cwd;

my ( $mock_npg_api_instrument, $model );

BEGIN {
  $mock_npg_api_instrument = Test::MockObject->new();
  $mock_npg_api_instrument->set_bound( q{model}, \$model );
  $mock_npg_api_instrument->fake_module( qw{ npg::api::instrument } );
  $mock_npg_api_instrument->fake_new( qw{ npg::api::instrument } );

  use_ok( q{npg_pipeline::archive::file::to_sra} );
}

use lib qw{t};
use t::util;

my $ua = t::useragent->new({ is_success => 1 });
my $util = t::util->new({
  useragent => $ua,
});
$util->catch_email();

my $curdir = getcwd();

$ENV{TEST_API_UTIL} = $util->apiutil();
$ENV{TEST_BIN} = $curdir . q{/t/bin/};
my $tmp_dir = $util->temp_directory();
$ENV{TEST_DIR} = $tmp_dir;
$ENV{NPG_WEBSERVICE_CACHE_DIR} = $curdir . q{/t/data};

my $analysis_runfolder_path = $util->analysis_runfolder_path();

my $conf_path = $util->conf_path();

sub create_analysis {
  `rm -rf $tmp_dir/staging`;
  `mkdir -p $analysis_runfolder_path/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive`;
  `mkdir $analysis_runfolder_path/Config`;
  `cp t/data/Recipes/Recipe_GA2_37Cycle_PE_v6.1.xml $analysis_runfolder_path/`;
  `cp t/data/Recipes/TileLayout.xml $analysis_runfolder_path/Config/`;
  `ln -s Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto $analysis_runfolder_path/Latest_Summary`;

  my $archive_root = qq{$analysis_runfolder_path/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/};
  foreach my $i (1..8) {
    foreach my $extension (qw{fastq fastqcheck gcfreq md5}) {
      my $file = $archive_root . qq{1234_$i}.qq{.$extension};
      `touch $file`;
    }
  }

  `touch $archive_root/1234_7_human.fastq`;
  `touch $archive_root/1234_7_nonhuman.fastq`;

  return 1;
}

my $run_obj = npg::api::run->new({
  util   => $util->apiutil(),
  id_run => 1234,
 });


{
  my $sra_uploader;
  lives_ok {
    $sra_uploader = npg_pipeline::archive::file::to_sra->new({
      run_folder => q{123456_IL2_1234},
      run => $run_obj,
      timestamp => q{20090709-123456},
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{create uploader ok};
  isa_ok( $sra_uploader, q{npg_pipeline::archive::file::to_sra}, q{$sra_uploader} );

  create_analysis();

  $model = q{HiSeq};

  lives_ok {
    $sra_uploader->archive();
  } q{archive runs ok};


}
1;