# $Id: 20-archive_file_generation-fastq.t 12581 2011-02-11 12:26:13Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 24;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::generation::fastq');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{TEST_RUN_OBJECT} = npg::api::run->new({
  util => $util->apiutil(),
  id_run => 1234,
});

{
  my $fastq_generator;

  lives_ok { $fastq_generator = npg_pipeline::archive::file::generation::fastq->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{no croak creating $fastq_generator object};
  isa_ok($fastq_generator, q{npg_pipeline::archive::file::generation::fastq}, q{$fastq_generator});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->create_analysis({no_touch_files => 1});

  $ENV{TEST_API_UTIL} = $util->apiutil();

  my @jids;
  lives_ok { @jids = $fastq_generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 1, q{1 job id returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $fastq_generator->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J fastq_creation_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qseq2fastq.pl --md5 /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.fastq.md5 --run 1234 --fastqcheck /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.fastqcheck --filter --unique  > /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.fastq'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct} );
}

{
  my $fastq_generator;

  lives_ok { $fastq_generator = npg_pipeline::archive::file::generation::fastq->new({
    run_folder => q{123456_IL2_1234},
    use_bases => q{Y37,I6n},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{no croak creating $fastq_generator object};
  isa_ok($fastq_generator, q{npg_pipeline::archive::file::generation::fastq}, q{$fastq_generator});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->create_tag_qseq();

  $ENV{TEST_API_UTIL} = $util->apiutil();

  my @jids;
  lives_ok { @jids = $fastq_generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 2, q{2 job ids returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $fastq_generator->_generate_bsub_command($arg_refs) );

  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J fastq_creation_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qseq2fastq.pl --md5 /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastq.md5 --run 1234 --fastqcheck /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastqcheck --read t --filter --unique /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_2_1_qseq_custom.txt > /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastq'};

  is($bsub_command, $expected_command, q{generated bsub command is correct});
}

{
  my $fastq_generator;

  lives_ok { $fastq_generator = npg_pipeline::archive::file::generation::fastq->new({
    run_folder => q{123456_IL2_1234},
    use_bases => q{Y37,I6n},
    timestamp => q{20090709-123456},
    lane => 4,
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{no croak creating $fastq_generator object};
  isa_ok($fastq_generator, q{npg_pipeline::archive::file::generation::fastq}, q{$fastq_generator});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };
  $util->create_tag_qseq();

  $ENV{TEST_API_UTIL} = $util->apiutil();

  my @jids;
  lives_ok { @jids = $fastq_generator->generate($arg_refs); } q{no croak running generate with a lane provided};
  is(scalar@jids, 2, q{2 job ids returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $fastq_generator->_generate_bsub_command($arg_refs) );

  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J fastq_creation_1234_20090709-123456[4] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qseq2fastq.pl --md5 /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastq.md5 --run 1234 --fastqcheck /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastqcheck --read t --filter --unique /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_2_1_qseq_custom.txt > /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastq'};
  is($bsub_command, $expected_command, q{generated bsub command is correct});
}

{
  my $fastq_generator;

  lives_ok { $fastq_generator = npg_pipeline::archive::file::generation::fastq->new({
    run_folder => q{123456_IL2_1234},
    use_bases => q{Y76,I8,y76},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{no croak creating $fastq_generator object};

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };
  $util->create_multiplex_analysis();

  $ENV{TEST_API_UTIL} = $util->apiutil();

  my @jids;
  lives_ok { @jids = $fastq_generator->generate($arg_refs); } q{no croak running generate with a lane provided};
  is(scalar@jids, 3, q{3 job ids returned});

  $arg_refs->{type_of_lanes} = q{indexed_lanes};

  my $bsub_command = $util->drop_temp_part_from_paths( $fastq_generator->_generate_bsub_command($arg_refs) );

  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J fastq_creation_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qseq2fastq.pl --md5 /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastq.md5 --run 1234 --fastqcheck /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastqcheck --index --read 2 --filter --unique /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_3_qseq_custom.txt > /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastq'};

  is( $bsub_command, $expected_command, q{bsub command as expected} );
}

{
  my $fastq_generator;

  lives_ok { $fastq_generator = npg_pipeline::archive::file::generation::fastq->new({
    run_folder => q{123456_IL2_1234},
    use_bases => q{Y76,I8,y76},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
  }); } q{no croak creating $fastq_generator object};

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };
  $util->create_multiplex_analysis();

  my $runfolder_path = $fastq_generator->runfolder_path();
#  my $specific_tag_file = qq{$runfolder_path/lane_8.tag};
  foreach my $lane ( 1..7 ) {
    qx{touch $runfolder_path/lane_$lane.tag};
  }
  qx[cp t/data/tag_files/lane_8.tag $runfolder_path/lane_8.tag];


  $ENV{TEST_API_UTIL} = $util->apiutil();

  my @jids;
  lives_ok { @jids = $fastq_generator->generate($arg_refs); } q{no croak running generate with a lane provided};
  is(scalar@jids, 6, q{6 job ids returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $fastq_generator->_generate_bsub_command($arg_refs) );

  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J fastq_creation_1234_20090709-123456[1-7] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qseq2fastq.pl --md5 /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastq.md5 --run 1234 --fastqcheck /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastqcheck --read 2 --filter --unique /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_3_qseq_custom.txt > /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastq'};
  is( $bsub_command, $expected_command, q{bsub command as expected - lanes 1->7 are not including} );
}

1;