# $Id: 30-run_folder-link.t 12581 2011-02-11 12:26:13Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 15;
use Test::Exception;
use lib qw{t};
use Digest::MD5;
use t::util;

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{TEST_FS_RESOURCE} = q{nfs_12};
my $tmp_dir = $util->temp_directory();
use_ok('npg_pipeline::run::folder::link');

{
  my $rfl;
  throws_ok {
    $rfl = npg_pipeline::run::folder::link->new({
      run_folder => q{123456_IL2_1234},
    });
  } qr{Attribute[ ][(]analysis_type[)][ ]is[ ]required}, q{missing analysis_type attribute};
}
{
  my $rfl;
  lives_ok {
    $rfl = npg_pipeline::run::folder::link->new({
      analysis_type => q{single},
      run_folder => q{123456_IL2_1234},
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak with all attributes provided - analysis_type => single};
  $util->set_single_analysis_area();
  
  lives_ok {
    $rfl->make_link();
    1;
  } q{no croak creating link for single};

  lives_ok {
    $rfl = npg_pipeline::run::folder::link->new({
      analysis_type => q{full},
      run_folder => q{123456_IL2_1234},
      analysis_path => q{Data/Intensities/Bustard1.3.4_10-07-2009_auto/GERALD_09-07-2009_auto},
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak with all attributes provided - analysis_type => full, analysis_path provided to ensure override of existing link};
  `mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.3.4_10-07-2009_auto/GERALD_09-07-2009_auto`;
  `touch $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.3.4_10-07-2009_auto/GERALD_09-07-2009_auto/touch_file`;
  lives_ok {
    $rfl->make_link();
    1;
  } q{no croak creating link - overriding single to auto};

  lives_ok {
    $rfl->make_link();
    1;
  } q{no croak creating link - no change since link already auto};

  `mv $tmp_dir/staging/IL2/analysis $tmp_dir/staging/IL2/outgoing`;
  $rfl->_set_folder(q{outgoing});
  lives_ok {
    $rfl->make_link();
    1;
  } q{no croak creating link - changed as folder route has changed};

  `rm -f $tmp_dir/staging/IL2/outgoing/123456_IL2_1234/Latest_Summary`;
  `mkdir -p $tmp_dir/staging/IL2/outgoing/123456_IL2_1234/Data/Intensities/Bustard_auto/GERALD_auto`;
  `touch $tmp_dir/staging/IL2/outgoing/123456_IL2_1234/Data/Intensities/Bustard_auto/GERALD_auto/touch_file`;
  $rfl->{analysis_path} = q{Data/Intensities/Bustard_auto/GERALD_auto};
  lives_ok {
    $rfl->make_link();
    1;
  } q{no croak creating link - changed as folder now using specified route};
}
{
  my $rfl;
  eval {
    $rfl = npg_pipeline::run::folder::link->new({
      analysis_type => q{single},
      run_folder => q{123456_IL2_1234},
      timestamp => q{20090709-123456},
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };
  my $bsub_command = $rfl->_generate_bsub_command($arg_refs);
  my $md5_object = Digest::MD5->new();
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );
  my $expected_bsub_cmd = q{bsub -P pipeline -w'done(123) && done(321)' -J create_latest_summary_link_123456_IL2_1234_20090709-123456  -R 'rusage[nfs_12=1]' -o /staging/IL2/outgoing/123456_IL2_1234/create_latest_summary_link_123456_IL2_1234_20090709-123456.out 't/bin/software/solexa/bin/create_summary_link --analysis_type single --conf_path data/config_files --domain test --folder analysis --run_folder 123456_IL2_1234 --verbose '};
  is( $bsub_command, $expected_bsub_cmd, q{bsub command ok when analysis dir to be used} );

  $rfl->_set_folder(q{outgoing});
  $bsub_command = $rfl->_generate_bsub_command($arg_refs);
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );
  $expected_bsub_cmd = q{bsub -P pipeline -w'done(123) && done(321)' -J create_latest_summary_link_123456_IL2_1234_20090709-123456  -R 'rusage[nfs_12=1]' -o /staging/IL2/outgoing/123456_IL2_1234/create_latest_summary_link_123456_IL2_1234_20090709-123456.out 't/bin/software/solexa/bin/create_summary_link --analysis_type single --conf_path data/config_files --domain test --folder outgoing --fs_resource nfs_12 --internal_script_bin t/bin/software/solexa/bin --run_folder 123456_IL2_1234 --runfolder_path /staging/IL2/outgoing/123456_IL2_1234 --verbose '};

  is( $bsub_command, $expected_bsub_cmd, q{bsub command ok when outgoing dir to be used} );

  lives_ok {
    $rfl->submit_create_link($arg_refs);
  } q{job submission successful};
  `rm -rf $tmp_dir/staging`;
  `mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.4.0_17-07-2009_ts6/GERALD_ts6`;
  lives_ok { qx[bin/create_summary_link --conf_path $conf_path --domain test --internal_script_bin /software/solexa/bin --verbose  --analysis_type single --timestamp 20090709-123456 --run_folder 123456_IL2_1234 --log_file no log file --folder outgoing]; } q{no croak running create_summary_link};
  ok(!$CHILD_ERROR, qq{Error code ok : $CHILD_ERROR});
}

{
  my $rfl;
  my $run_folder = q{123456_IL2_1234};
  eval {
    $rfl = npg_pipeline::run::folder::link->new({
      analysis_type => q{single},
      run_folder => $run_folder,
      timestamp => q{20090709-123456},
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };
  `rm -rf $tmp_dir/staging`;
  `mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.4.0_17-07-2009_ts6/GERALD_ts6`;
  `mkdir -p $tmp_dir/staging/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard1.4.0_23-07-2009_dj3/GERALD_dj3`;

  my $inst_dir   = $rfl->_get_instrument_dir($run_folder);
  $inst_dir =~ s/incoming/analysis/xms;
  throws_ok {
    $rfl->make_link();
    1;
  } qr{link[ ]command[ ]failed[ ]-[ ]1234: found[ ]multiple[ ]possible[ ]recalibrated_directories}, q{croak creating link};  
}
1;
