# $Id: 30-run_status-update.t 12581 2011-02-11 12:26:13Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 13;
use Test::Exception;
use lib qw{t};
use t::util;
#use SangerPaths qw{trackingnpg};
use npg::api::run;
use Digest::MD5;
use File::Slurp;

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

use_ok('npg_pipeline::run::status::update');

my $run_folder = $util->test_run_folder();

{
  my $rsu;
  throws_ok {
    $rsu = npg_pipeline::run::status::update->new({
      run    => 'run object',
      id_run => 1234,
      run_folder => $run_folder,
    });
  } qr/Attribute[ ][(]status_one[)][ ]is[ ]required/, q{missing status_one attribute};
}
{
  my $rsu;
  eval {
    $rsu = npg_pipeline::run::status::update->new({
      status_one => 'run archived',
      id_run     => 1234,
      run_folder => $run_folder,
      conf_path => $conf_path,
    });
  };
  isa_ok($rsu, q{npg_pipeline::run::status::update}, q{$rsu});
}
{

  $util->set_rta_staging_analysis_area();

  my $rsu;
  eval {
    $rsu = npg_pipeline::run::status::update->new({
      run        => 'run object',
      status_one => 'run archived',
      id_run     => 1234,
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  throws_ok {
    $rsu->status_one('test');
  } qr/Cannot[ ]assign[ ]a[ ]value[ ]to[ ]a[ ]read-only[ ]accessor/, q{cannot assign to a read_only accessor - status_one};
  throws_ok {
    $rsu->run('test');
  } qr/Cannot[ ]assign[ ]a[ ]value[ ]to[ ]a[ ]read-only[ ]accessor/, q{cannot assign to a read_only accessor - run};
  throws_ok {
    $rsu->id_run('4321');
  } qr/Cannot[ ]assign[ ]a[ ]value[ ]to[ ]a[ ]read-only[ ]accessor/, q{cannot assign to a read_only accessor - id_run};
  is($rsu->id_run(), 1234, q{id_run value still correct});

}
{

  $util->set_rta_staging_analysis_area();


  my $rsu;
  eval {
    $rsu = npg_pipeline::run::status::update->new({
      run        => 'run object',
      status_one => 'run archived',
      id_run     => 1234,
      _util      => q{hello world},
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      conf_path => $conf_path,
      domain => q{test},
    });
  };
}
{
  my $rsu;
  my $api_run = npg::api::run->new({
    util   => $util->apiutil(),
    id_run => 1234,
  });
  eval {
    $rsu = npg_pipeline::run::status::update->new({
      run        => $api_run,
      status_one => 'run archived',
      id_run     => 1234,
      run_folder =>$run_folder,
      timestamp  => q{20090709-123456},
      runfolder_path => $util->analysis_runfolder_path(),
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
    });
  };
  is($rsu->run(), $api_run, q{fake api_run in use});
  isa_ok($rsu->run()->util()->useragent(), q{t::useragent}, q{fake useragent in place});
  lives_ok {
    $rsu->update();
  } q{no croak updating};

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->remove_staging();
  my $bsub_command = $util->drop_temp_part_from_paths( $rsu->_generate_bsub_command($arg_refs) );
  my $expected_cmd =  q{bsub -w'done(123) && done(321)' -J update_run_status_1234_run_archived_20090709-123456 -q small -R 'rusage[nfs_12=1]' -o /staging/IL2/outgoing/123456_IL2_1234/update_run_status_1234_run_archived_20090709-123456.out 't/bin/software/solexa/bin/npg_pipeline_update_run_status --conf_path "data/config_files" --domain "test" --id_run "1234" --run_folder "123456_IL2_1234" --runfolder_path "/staging/IL2/analysis/123456_IL2_1234" --status_one "run archived" --verbose '};

  is($bsub_command, $expected_cmd, q{bsub command ok when updating status to "run archived"});

  $util->set_rta_staging_analysis_area();

}
my $runfolder_path = $util->analysis_runfolder_path();
$ENV{TEST_USERAGENT} = $runfolder_path . q{/useragent};

write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

{
  my $output;
  lives_ok { $output = qx[bin/npg_pipeline_update_run_status --conf_path $conf_path --domain test --verbose  --run_folder "123456_IL2_1234" --id_run "1234" --status_one "run archived" --runfolder_path $runfolder_path]; } q{update_status ran};
  ok(!$CHILD_ERROR, qq{Error code ok : $CHILD_ERROR});

  diag $output;
}

$util->remove_staging();

1;